# Generated by Django 3.2.5 on 2022-11-18 12:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bank_sync', '0012_alter_ipn_account_number'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ipndata',
            name='ipn',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bank_sync.ipn'),
        ),
    ]



from django.db import models
from django.utils import timezone


class Callbacks(models.Model):
    bank_id = models.IntegerField()
    type_code = models.IntegerField()
    reference = models.CharField(max_length=100, unique=True)
    callback = models.TextField()
    request = models.JSONField(default=dict)
    response = models.JSONField(default=dict)
    request_time = models.DateTimeField(default=timezone.now)
    response_time = models.DateTimeField(default=timezone.now)

    class Meta:
        verbose_name = "Callback"
        verbose_name_plural = "Callbacks"

    def __str__(self):
        return self.reference

    def __repr__(self):
        return f'Callbacks(reference="{self.reference}",callback="{self.callback}", request={self.request}, response={self.response})'


class RequestLogs(models.Model):
    ip = models.CharField(max_length=50)
    method = models.CharField(max_length=10)
    path = models.CharField(max_length=100)
    content_type = models.CharField(max_length=50)
    content_params = models.JSONField(default=dict)
    body = models.JSONField(default=dict)
    headers = models.JSONField(default=dict)
    user = models.CharField(max_length=100)
    error = models.TextField(default=None, null=True)
    log_time = models.DateTimeField(default=timezone.now)

    class Meta:
        verbose_name = "Request Log"
        verbose_name_plural = "Request Logs"

    def __str__(self):
        return f'{self.log_time.strftime("%d %B %Y - %H:%M:%S")} {self.ip} {self.method} {self.path}'

    def __repr__(self):
        return f'RequestLogs(ip="{self.ip}", method="{self.method}", path="{self.path}", content_type="{self.content_type}", content_params="{self.content_params}", body="{self.body}", headers="{self.headers}", user="{self.user}")'


class ResponseLogs(models.Model):
    ip = models.CharField(max_length=50, default=None, null=True)
    headers = models.JSONField(default=dict)
    content = models.JSONField(default=dict)
    log_time = models.DateTimeField(default=timezone.now)
    error = models.TextField(default=None, null=True)

    def __str__(self):
        return f'{self.log_time.strftime("%d %B %Y - %H:%M:%S")} {self.ip}'

    def __repr__(self):
        return f'ResponseLogs(headers="{self.headers}", content="{self.content}")'

    class Meta:
        verbose_name = "Response Log"
        verbose_name_plural = "Response Logs"

# This class is used to create/save user I.P.N. callbacks that are to listen on any changes on a specified bank and account number


class IPN(models.Model):
    client_id = models.CharField(max_length=200, unique=True)
    signature = models.CharField(max_length=200, unique=True)
    bank_id = models.IntegerField()
    country_code = models.CharField(max_length=5, default='KE')
    account_number = models.CharField(max_length=200, unique=True)
    url = models.URLField()

    IPNSTATUS = [
        (True, 'Active'),
        (False, 'Inactive')
    ]
    status = models.BooleanField(choices=IPNSTATUS, default=True)

    IPNTYPE = [
        (2, 'Debit'),
        (1, 'Credit'),
        (0, 'Both')
    ]

    ipn_type = models.IntegerField(choices=IPNTYPE,
                                   default=0,)

    def __str__(self):
        return f'{self.client_id} : I.P.N. Type - {[i[1] for i in self.IPNTYPE if i[0] == self.ipn_type][0]} : Status - {[i[1] for i in self.IPNSTATUS if i[0] == self.status][0]}'

    def __repr__(self):
        return f'IPN(client_id="{self.client_id}", bank_id="{self.bank_id}", country_code="{self.country_code}", account_number="{self.account_number}", ipn_type={self.ipn_type}, status={self.status})'


# This class is used to create/save user I.P.N. data sent back
class IPNData(models.Model):
    ipn = models.ForeignKey(IPN, on_delete=models.CASCADE)
    response = models.JSONField(default=dict)
    response_time = models.DateTimeField(default=timezone.now)

    class Meta:
        verbose_name = "IPNData"
        verbose_name_plural = "IPNData"

    def __str__(self):
        return str(self.ipn)

    def __repr__(self):
        return f'IPNData(ipn_pk="{self.ipn.pk}", response={self.response})'



class Operations(object):

    # These are the banking integrations
    # Each with a unique ID
    GLOBAL = 0
    COOP = GLOBAL + 1
    EQUITY = COOP + 1
    NCBA = EQUITY + 1
    DAPI = NCBA + 1
    MONO = DAPI + 1
    PLAID = MONO + 1
    STITCH = PLAID + 1
    DTB = STITCH + 1
    UBA = DTB + 1
    KCB = UBA + 1
    SCB = KCB + 1
    SAFCOM = SCB + 1
    FAULU = SAFCOM + 1
    CONSOLIDATED = FAULU + 1
    STANBIC = CONSOLIDATED + 1
    FAMILY=STANBIC + 1
    NCBA_LOOP=FAMILY + 1

    # These are the operations to be performed in Banking Identity
    # Each with a unique incremental ID

    # Returns Sample Data
    OPERATIONS = -2
    SAMPLE_DATA = OPERATIONS + 1
    BANKS = SAMPLE_DATA + 1
    BALANCE = BANKS + 1
    MINI_STATEMENT = BALANCE + 1
    FULL_STATEMENT = MINI_STATEMENT + 1
    ACCOUNT_VALIDATION = FULL_STATEMENT + 1
    ACCOUNT_TRANSACTIONS = ACCOUNT_VALIDATION + 1
    IFT = ACCOUNT_TRANSACTIONS + 1
    TRANSACTION_STATUS = IFT + 1
    MOBILE_WALLET = TRANSACTION_STATUS + 1
    PESALINK_TO_BANK = MOBILE_WALLET + 1
    PESALINK_TO_MOBILE = PESALINK_TO_BANK + 1
    RTGS = PESALINK_TO_MOBILE + 1
    SWIFT = RTGS + 1
    EFT = SWIFT + 1
    OMN = EFT + 1
    INITIATE_PAYMENT = OMN + 1
    FOREX_RATE = INITIATE_PAYMENT + 1
    COUNTRIES = FOREX_RATE + 1
    STK_PUSH = COUNTRIES + 1
    BANKS_BY_CODE = STK_PUSH + 1
    BANKS_BY_ID = BANKS_BY_CODE + 1
    PDF_TO_JSON = BANKS_BY_ID + 1
    GET_JSON_PDF = PDF_TO_JSON + 1
    IPN_REGISTER = GET_JSON_PDF + 1
    IPN_GET = IPN_REGISTER + 1
    IPN_UPDATE = IPN_GET + 1
    IPN_DELETE = IPN_UPDATE + 1
    IPN_GENERATE_CLIENT_SECRET = IPN_DELETE + 1
    JOBS_ACCOUNT_BALANCES = IPN_GENERATE_CLIENT_SECRET + 1
    JOBS_ACCOUNT_STATEMENT = JOBS_ACCOUNT_BALANCES + 1

    # All end points are placed in the BANKS_CONF
    # This is to allow easy standardization of the base url, endpoint and operations
    # When calling the APIs, in this case this is done in the resource class read function
    BANKS_CONF = {
        GLOBAL: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "read": {
                "endpoint": f"api/banks",
                "operations": {
                    OPERATIONS: ("operations", "GET"),
                    SAMPLE_DATA: ("sample_payload", "POST"),
                    BANKS: ("banks", "GET"),
                    BANKS_BY_CODE: ("banks/by_code", "GET"),
                    BANKS_BY_ID: ("banks/by_id", "GET"),
                    COUNTRIES: ("countries", "GET"),
                    IPN_REGISTER: ("ipn/register", "POST"),
                    IPN_GET: ("ipn/get", "POST"),
                    IPN_UPDATE: ("ipn/update", "POST"),
                    IPN_DELETE: ("ipn/delete", "POST"),
                    IPN_GENERATE_CLIENT_SECRET: ("ipn/generate_client_secret", "POST"),
                    JOBS_ACCOUNT_BALANCES: ("jobs/account_balances", "POST"),
                    JOBS_ACCOUNT_STATEMENT: ("jobs/account_statement", "POST"),
                }
            }

        },
        UBA: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{UBA}",
                "operations": {
                    BALANCE: ("get_balance", "POST"),
                    FULL_STATEMENT: ("get_full_statement", "POST"),
                }
            },
            "payment": {
                "endpoint": f"api/banks/payment/{UBA}",
                "operations": {
                    IFT: ("ift", "POST"),
                }
            }

        },
        DTB: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{DTB}",
                "operations": {
                    ACCOUNT_VALIDATION: ("get_validation", "POST"),
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {
                "endpoint": f"api/banks/payment/{DTB}",
                "operations": {
                    IFT: ("ift", "POST"),
                    MOBILE_WALLET: ("mobile_wallet", "POST"),
                    TRANSACTION_STATUS: ("get_transaction_status", "POST"),
                    STK_PUSH: ("c2b_stk_push", "POST"),
                }
            }

        },
        EQUITY: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{EQUITY}",
                "operations": {
                    BALANCE: ("get_balance", "POST"),
                    MINI_STATEMENT: ("get_mini_statement", "POST"),
                    ACCOUNT_VALIDATION: ("get_validation", "POST"),
                    FULL_STATEMENT: ("get_full_statement", "POST"),
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                    FOREX_RATE: ("forex_rate", "POST")
                }
            },
            "payment": {
                "endpoint": f"api/banks/payment/{EQUITY}",
                "operations": {
                    IFT: ("ift", "POST"),
                    MOBILE_WALLET: ("mobile_wallet", "POST"),
                    RTGS: ("rtgs", "POST"),
                    SWIFT: ("swift", "POST"),
                    EFT: ("eft", "POST"),
                    PESALINK_TO_BANK: ("pesalink/bank", "POST"),
                    PESALINK_TO_MOBILE: ("pesalink/mobile", "POST"),
                    TRANSACTION_STATUS: ("get_transaction_status", "POST"),
                }
            }

        },
        COOP: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{COOP}",
                "operations": {
                    BALANCE: ("get_balance", "POST"),
                    MINI_STATEMENT: ("get_mini_statement", "POST"),
                    ACCOUNT_VALIDATION: ("get_validation", "POST"),
                    FULL_STATEMENT: ("get_full_statement", "POST"),
                    ACCOUNT_TRANSACTIONS: ("get_transactions", "POST"),
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {
                "endpoint": f"api/banks/payment/{COOP}",
                "operations": {
                    IFT: ("ift", "POST"),
                    MOBILE_WALLET: ("mobile_wallet", "POST"),
                    PESALINK_TO_BANK: ("pesalink/bank", "POST"),
                    PESALINK_TO_MOBILE: ("pesalink/mobile", "POST"),
                    TRANSACTION_STATUS: ("get_transaction_status", "POST"),
                }
            }

        },
        NCBA: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{NCBA}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {
                "endpoint": f"api/banks/payment/{NCBA}",
                "operations": {
                    IFT: ("ift", "POST"),
                    MOBILE_WALLET: ("mobile_wallet", "POST"),
                    RTGS: ("rtgs", "POST"),
                    EFT: ("eft", "POST"),
                    PESALINK_TO_BANK: ("pesalink/bank", "POST"),
                    TRANSACTION_STATUS: ("get_transaction_status", "POST"),
                }
            }
        },
        SAFCOM: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{SAFCOM}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        SCB: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{SCB}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        KCB: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{KCB}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        FAULU: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{FAULU}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        CONSOLIDATED: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{CONSOLIDATED}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        STANBIC: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{STANBIC}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        FAMILY: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{FAMILY}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
        NCBA_LOOP: {
            "url": "https://dev.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/banks/accounts/{NCBA_LOOP}",
                "operations": {
                    PDF_TO_JSON: ("pdf_to_json", "POST"),
                    GET_JSON_PDF: ("uploaded_pdf_json", "POST"),
                }
            },
            "payment": {}
        },
    }

    THIRD_PARTY_BANKING = {
        DAPI: {
            "url": "https://dev.3p.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/v1/Dapi",
                "operations": {
                    BALANCE:  ("get_balance", "POST"),
                    ACCOUNT_TRANSACTIONS: ("get_transactions", "POST")
                }                
            },
            "payment": {
                "endpoint": f"api/v1/Dapi",
                "operations": {
                    INITIATE_PAYMENT: ("initiate_payment", "POST"),
                }
            }
        },
        MONO: {
            "url": "https://dev.3p.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/v1/Mono",
                "operations": {
                    BALANCE:  ("get_balance", "POST"),
                    ACCOUNT_TRANSACTIONS: ("get_transactions", "POST")
                }                
            },
            "payment": {
                "endpoint": f"api/v1/Mono",
                "operations": {
                    INITIATE_PAYMENT: ("initiate_payment", "POST"),
                }
            }
        },
        PLAID: {
            "url": "https://dev.3p.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/v1/Plaid",
                "operations": {
                    BALANCE:  ("get_balance", "POST"),
                    ACCOUNT_TRANSACTIONS: ("get_transactions", "POST")
                }                
            },
            "payment": {
                "endpoint": f"api/v1/Plaid",
                "operations": {
                    INITIATE_PAYMENT: ("initiate_payment", "POST"),
                }
            }
        },
        STITCH: {
            "url": "https://dev.3p.bankingservices.nashglobal.co",
            "accounts": {
                "endpoint": f"api/v1/Stitch",
                "operations": {
                    BALANCE:  ("get_balance", "POST"),
                    ACCOUNT_TRANSACTIONS: ("get_transactions", "POST")
                }                
            },
            "payment": {
                "endpoint": f"api/v1/Stitch",
                "operations": {
                    INITIATE_PAYMENT: ("initiate_payment", "POST"),
                }
            }
        },
    }



# Generated by Django 3.2.5 on 2022-11-17 11:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bank_sync', '0009_responselogs_error'),
    ]

    operations = [
        migrations.CreateModel(
            name='IPN',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('client_id', models.CharField(max_length=200, unique=True)),
                ('client_secret', models.CharField(max_length=200, unique=True)),
                ('bank_id', models.IntegerField()),
                ('account_number', models.CharField(max_length=200)),
                ('url', models.URLField()),
                ('status', models.BooleanField(choices=[(True, 'Active'), (False, 'Inactive')], default=True)),
                ('ipn_type', models.IntegerField(choices=[(2, 'Debit'), (1, 'Credit'), (0, 'Both')], default=0)),
            ],
        ),
        migrations.AlterModelOptions(
            name='callbacks',
            options={'verbose_name': 'Callback', 'verbose_name_plural': 'Callbacks'},
        ),
        migrations.AlterModelOptions(
            name='requestlogs',
            options={'verbose_name': 'Request Log', 'verbose_name_plural': 'Request Logs'},
        ),
        migrations.AlterModelOptions(
            name='responselogs',
            options={'verbose_name': 'Response Log', 'verbose_name_plural': 'Response Logs'},
        ),
    ]



import requests

_client_id = "NashSyncERP"
_client_secret = "secret"

_payload = {
    'grant_type': 'client_credentials',
    'scope': "NashSyncERP TreasuryService",
    'client_id': _client_id,
    'client_secret': _client_secret
}


_auth_token_url = "https://authserver.purplecliff-03d4fbdd.westeurope.azurecontainerapps.io/connect/token"

_identity_token = requests.request("POST", _auth_token_url, data=_payload)

print('=======================================================')
print(f'_identity_token angular purple: {_identity_token.text}')
print('=======================================================')

# print()
# print()
# print()

# _auth_token_url="https://identity.nashglobal.co/connect/token"

# _identity_token = requests.request("POST", _auth_token_url, data=_payload)

# print('=======================================================')
# print(f'_identity_token: {_identity_token.text}')
# print('=======================================================')
